<?php

namespace Drupal\acquia_dam;

use Drupal\acquia_dam\Client\AcquiaDamClientFactory;
use Drupal\Core\Database\Connection;
use Drupal\Core\DestructableInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Logger\LoggerChannelInterface;

/**
 * Register and remove integration links from Acquia DAM.
 */
class IntegrationLinkRegister implements DestructableInterface {

  /**
   * List of entities to create integration links for.
   *
   * @var array
   */
  protected $integrations;

  /**
   * List of entities to delete associated integration links.
   *
   * @var array
   */
  protected $integrationsToDelete;


  /**
   * List of integration links to delete.
   *
   * @var array
   */
  protected $trackingToDelete;

  /**
   * DAM client factory.
   *
   * @var \Drupal\acquia_dam\Client\AcquiaDamClientFactory
   */
  protected $clientFactory;

  /**
   * Database connection service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Entity type bundle info.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityBundleInfo;

  /**
   * Logger channel for Acquia DAM.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $damLoggerChannel;

  /**
   * IntegrationLinkRegister constructor.
   *
   * @param \Drupal\acquia_dam\Client\AcquiaDamClientFactory $client_factory
   *   DAM client factory.
   * @param \Drupal\Core\Database\Connection $database
   *   Database connection.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $loggerChannel
   *   Logger channel instance.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entityBundleInfo
   *   Entity bundle info.
   */
  public function __construct(AcquiaDamClientFactory $client_factory, Connection $database, LoggerChannelInterface $loggerChannel, EntityTypeBundleInfoInterface $entityBundleInfo) {
    $this->clientFactory = $client_factory;
    $this->database = $database;
    $this->damLoggerChannel = $loggerChannel;
    $this->entityBundleInfo = $entityBundleInfo;
  }

  /**
   * Add entity and necessary info to the list for integration link creation.
   *
   * @param string $asset_id
   *   DAM asset id.
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   Content entity instance.
   */
  public function addIntegrationLinksList(string $asset_id, ContentEntityInterface $entity): void {
    $this->integrations[$entity->uuid()]['asset_uuids'][] = $asset_id;
    $this->integrations[$entity->uuid()]['entity'] = $entity;
  }

  /**
   * Add entity to the deletion list.
   *
   * @param string $entity_uuid
   *   Entity uuid.
   */
  public function addIntegrationToDeleteList(string $entity_uuid) {
    $this->integrationsToDelete[] = $entity_uuid;
  }

  /**
   * Add asset to tracking delete list.
   *
   * @param string $asset_uuid
   *   Asset uuid.
   * @param string $entity_uuid
   *   Entity uuid.
   */
  public function addToTrackingDeleteList(string $asset_uuid, string $entity_uuid) {
    $this->trackingToDelete[$entity_uuid][] = $asset_uuid;
  }

  /**
   * Register integration links from integrations list.
   */
  protected function registerIntegrationLinks(): void {
    foreach ($this->integrations as $item) {
      foreach ($item['asset_uuids'] as $asset_uuid) {
        $type = $item['entity']->getEntityTypeId();
        $this->registerIntegrationLink($asset_uuid, $item['entity']);
      }
    }

    $this->integrations = [];
  }

  /**
   * Remove integration list based on integrationsToDelete list.
   */
  protected function removeIntegrationLinks(): void {
    foreach ($this->integrationsToDelete as $entity_uuid) {
      $this->removeIntegrationLink($entity_uuid);
    }
  }

  /**
   * Remove integration list based on integrationsToDelete list.
   */
  protected function removeTracking(): void {
    foreach ($this->trackingToDelete as $entity_uuid => $asset_ids) {
      $this->removeTrackings($entity_uuid, $asset_ids);
    }
  }

  /**
   * Add integration link for a given entity.
   *
   * @param string $asset_id
   *   DAM asset id.
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   Entity object.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  protected function registerIntegrationLink(string $asset_id, ContentEntityInterface $entity): void {
    try {
      $url = $entity->toUrl()->setAbsolute()->toString();
      $client = $this->clientFactory->getUserClient();
      $data = $client->registerIntegrationLink($asset_id, $url, $this->generateIntegrationLinkDescription($entity));
      $this->trackIntegration($asset_id, $entity->uuid(), $entity->getEntityTypeId(), $data['uuid'], $url);
    }
    catch (\Exception $exception) {
      $this->damLoggerChannel->error(sprintf(
        'Something went wrong during integration link registration for asset: %s. Error message: %s',
        $asset_id,
        $exception->getMessage()
      ));
    }
  }

  /**
   * Remove integration links for given entity.
   *
   * @param string $entity_uuid
   *   Entity uuid.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  protected function removeIntegrationLink(string $entity_uuid): void {
    try {
      $client = $this->clientFactory->getUserClient();

      $integration_links = $this
        ->database
        ->select('acquia_dam_integration_link_tracking', 'int')
        ->fields('int', ['integration_link_id'])
        ->condition('entity_uuid', $entity_uuid)
        ->execute()
        ->fetchAll();

      if (empty($integration_links)) {
        $this->damLoggerChannel->warning(sprintf(
          'There are no integration links associated with entity: %s in local storage',
          $entity_uuid,
        ));
      }

      foreach ($integration_links as $integration_link) {
        $client->removeIntegrationLink($integration_link->integration_link_id);
      }

      $this->removeIntegrationTracking($entity_uuid);
      $this->integrationsToDelete = [];
    }
    catch (\Exception $exception) {
      $this->damLoggerChannel->error(sprintf(
        'Something went wrong during integration link removal for entity: %s integration link: %s. Error message: %s',
        $entity_uuid,
        $integration_link->integration_link_id,
        $exception->getMessage()
      ));
    }
  }

  /**
   * Remove integration links by integration link ids.
   *
   * @param string $entity_uuid
   *   Entity instance where the assets are used.
   * @param array $asset_ids
   *   Asset ids.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  protected function removeTrackings(string $entity_uuid, array $asset_ids) {
    try {
      $client = $this->clientFactory->getUserClient();

      $integration_links = $this
        ->database
        ->select('acquia_dam_integration_link_tracking', 'int')
        ->fields('int', ['integration_link_id'])
        ->condition('entity_uuid', $entity_uuid)
        ->condition('asset_uuid', $asset_ids, 'IN')
        ->execute()
        ->fetchAll();

      if (empty($integration_links)) {
        $this->damLoggerChannel->warning(sprintf(
          'There are no integration links associated with entity: %s in local storage',
          $entity_uuid,
        ));
      }

      foreach ($integration_links as $integration_link) {
        $client->removeIntegrationLink($integration_link->integration_link_id);
        $this
          ->database
          ->delete('acquia_dam_integration_link_tracking')
          ->condition('integration_link_id', $integration_link->integration_link_id)
          ->execute();
      }

      $this->trackingToDelete = [];
    }
    catch (\Exception $exception) {
      $this->damLoggerChannel->error(sprintf(
        'Something went wrong during integration link removal for entity: %s integration link: %s. Error message: %s',
        $entity_uuid,
        $integration_link->integration_link_id,
        $exception->getMessage()
      ));
    }
  }

  /**
   * Tracks integration link info in custom database table.
   *
   * @param string $asset_uuid
   *   Asset uuid.
   * @param string $entity_uuid
   *   Entity uuid.
   * @param string $entity_type
   *   Entity type id.
   * @param string $integration_id
   *   Integration link id.
   * @param string $path
   *   Path to entity.
   *
   * @throws \Exception
   */
  protected function trackIntegration(string $asset_uuid, string $entity_uuid, string $entity_type, string $integration_id, string $path) {
    $this->database->insert('acquia_dam_integration_link_tracking')
      ->fields([
        'asset_uuid' => $asset_uuid,
        'entity_uuid' => $entity_uuid,
        'entity_type' => $entity_type,
        'integration_link_id' => $integration_id,
        'path' => $path,
        'created' => date('c'),
      ])
      ->execute();
  }

  /**
   * Removes integration link tracking from custom database table.
   *
   * @param string $entity_uuid
   *   Entity uuid.
   */
  protected function removeIntegrationTracking(string $entity_uuid) {
    $this->database->delete('acquia_dam_integration_link_tracking')
      ->condition('entity_uuid', $entity_uuid)
      ->execute();
  }

  /**
   * Generates integration link description.
   *
   * Description can be 255 characters maximum.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   Entity instance.
   *
   * @return string
   *   Description for integration links.
   */
  protected function generateIntegrationLinkDescription(ContentEntityInterface $entity): string {
    $bundle_info = $this->entityBundleInfo->getAllBundleInfo();

    // Bundle label can be 50 chars maximum.
    $bundle_label = $bundle_info[$entity->getEntityTypeId()][$entity->bundle()]['label'];
    $bundle_label = strlen($bundle_label) <= 50 ?
      $bundle_label
      : substr($bundle_label, 0, 47) . '...';

    // Entity title can be 195 chars maximum.
    $label = $entity->label();
    $label = strlen($label) <= 195 ?
      $label
      : substr($label, 0, 192) . '...';

    return sprintf('%s titled "%s"', $bundle_label, $label);
  }

  /**
   * {@inheritdoc}
   */
  public function destruct() {
    // Start with delete to avoid errors on register triggered by title change.
    if (!empty($this->integrationsToDelete)) {
      $this->removeIntegrationLinks();
    }
    if (!empty($this->integrations)) {
      $this->registerIntegrationLinks();
    }
    if (!empty($this->trackingToDelete)) {
      $this->removeTracking();
    }
  }

}
